using WebAPI.Models;
using WebAPI.Services;
using Microsoft.AspNetCore.Mvc;
using System.ComponentModel.DataAnnotations;
using Microsoft.AspNetCore.Authorization;


namespace WebAPI.Controllers;

[ApiController]
[Route("[controller]")]
public class ExperienceController : ControllerBase
{
    private readonly ExperienceService _experienceService;

    public ExperienceController(ExperienceService experienceService)
    {
        _experienceService = experienceService;
    }

    // GET all experiences
    [HttpGet]
    public ActionResult<List<Experience>> GetAll()
    {
        var experiences = _experienceService.GetAll().Select(e => new
        {
            e.Name,
            e.Price,
        }).ToList();
        return Ok(experiences);
    }

    // GET all experiences in a specific shared experience
    [HttpGet("experience/{sharedExperienceName}")]
    public ActionResult<List<Experience>> GetExperiencesInSharedExperience(string sharedExperienceName)
    {
        var experiences = _experienceService.GetExperiencesInSharedExperience(sharedExperienceName)
        .Select(e => new
        {
            e.Name,
        }).ToList();
        return Ok(experiences);
    }

    // GET the minimum, average, and maximum price for the whole experience in the system.
    [HttpGet("experiencePrice")]
    [Authorize(Roles = "Manager,Admin")]
    public ActionResult<PriceStatistics> GetPrices()
    {
        var priceStats = _experienceService.GetPriceStatistics();
        return Ok(priceStats);
    }

    // GET the number of guests and sum of sales for each experience available in the system.
    [HttpGet("experienceSales")]
    [Authorize(Roles = "Manager,Admin")]
    public ActionResult<ExperienceSales> GetExperienceSales()
    {
        var experienceSales = _experienceService.GetExperienceSales();
        return Ok(experienceSales);
    }


    //CRUD-operations from assignment
    //1. Add a new experience.
    [HttpGet("{id}")]
    [Authorize(Roles = "Admin")]
    public ActionResult<Experience> GetExperienceById(int id)
    {
        var experience = _experienceService.GetExperienceById(id);
        return Ok(experience);
    }

    [HttpPost]
    [Authorize(Roles = "Admin")]
    public ActionResult<ExperienceDTO> AddExperience([FromBody] ExperienceDTO experienceDTO)
    {
        if (!ModelState.IsValid)
        {
            return BadRequest(ModelState);
        }

        var provider = _experienceService.GetProviderByName(experienceDTO.ProviderName);
        if (provider == null)
        {
            return BadRequest($"Provider '{experienceDTO.ProviderName}' not found.");
        }

        var experience = new Experience
        {
            Name = experienceDTO.Name,
            Price = Convert.ToInt16(experienceDTO.Price),
            Provider = provider
        };

        var newExperience = _experienceService.AddExperience(experience);

        // Create DTO for response
        var responseDTO = new ExperienceDTO
        {
            Name = newExperience.Name,
            Price = newExperience.Price,
            ProviderName = newExperience.Provider.Name
        };

        return CreatedAtAction(nameof(GetExperienceById), new { id = newExperience.ExperienceId }, responseDTO); // 201 Created
    }


    //2. Update the price of one of the experiences.
    [HttpPut("{experienceId}")]
    [Authorize(Roles = "Admin")]
    public ActionResult<Experience> UpdateExperiencePrice(int experienceId, [FromBody] ExperiencePriceUpdateDTO experiencePriceUpdateDTO)
    {
        //Checking if the price is valid using the NonNegativePriceAttribute
        if (!ModelState.IsValid)
        {
            return BadRequest(ModelState); // Return validation errors
        }

        var experience = _experienceService.GetExperienceById(experienceId);
        if (experience == null)
        {
            return NotFound($"Experience '{experienceId}' not found.");
        }

        // Update the price
        experience.Price = Convert.ToInt16(experiencePriceUpdateDTO.Price);
        _experienceService.UpdateExperiencePrice(experienceId, Convert.ToInt16(experiencePriceUpdateDTO.Price));

        return Ok(experience);
    }

    //3. Delete an experience
    [HttpDelete("{experienceId}")]
    [Authorize(Roles = "Admin")]
    public ActionResult DeleteExperience(int experienceId)
    {
        var experience = _experienceService.GetExperienceById(experienceId);
        if (experience == null)
        {
            return BadRequest($"Experience '{experienceId}' not found.");
        }

        _experienceService.DeleteExperience(experienceId);

        return NoContent(); // 204 No Content
    }
}

public class ExperienceDTO
{
    public string Name { get; set; }

    [NonNegativePrice]
    public double Price { get; set; }
    public string ProviderName { get; set; }
}

public class ExperiencePriceUpdateDTO
{
    [NonNegativePrice]
    public double Price { get; set; }
}

public class NonNegativePriceAttribute : ValidationAttribute
{
    protected override ValidationResult IsValid(object value, ValidationContext validationContext)
    {
        if (value is double price && price < 0)
        {
            return new ValidationResult("Price cannot be negative. Please provide a value of 0 or higher.");
        }
        return ValidationResult.Success;
    }
}
