DROP TABLE IF EXISTS ExperienceInSharedExperience;
DROP TABLE IF EXISTS GuestRegistrations;
DROP TABLE IF EXISTS Guests;
DROP TABLE IF EXISTS SharedExperiences;
DROP TABLE IF EXISTS Experiences;
DROP TABLE IF EXISTS ExperiencesOverview;
DROP TABLE IF EXISTS ExperienceProviders;



CREATE TABLE ExperienceProviders (
    ProviderId VARCHAR(20) PRIMARY KEY,       
    ProviderName VARCHAR(100) NOT NULL, 
    Address VARCHAR(255),              
    PhoneNumber VARCHAR(20)             
);

CREATE TABLE Experiences (
    ExperienceID INT IDENTITY(1,1) PRIMARY KEY,  
    ExperienceName VARCHAR(255) NOT NULL,        
    ExperienceProviderCVR VARCHAR(20),           
    Price DECIMAL(10,2),                         
    FOREIGN KEY (ExperienceProviderCVR) REFERENCES ExperienceProviders(ProviderId) 
);

CREATE TABLE SharedExperiences (
    SharedExperienceID INT IDENTITY(1,1) PRIMARY KEY,  
    ExperienceName VARCHAR(255) NOT NULL,             
    ExperienceDate DATE NOT NULL                      
);

CREATE TABLE Guests (
    GuestID INT IDENTITY(1,1) PRIMARY KEY,  
    Name VARCHAR(255) NOT NULL
);

CREATE TABLE GuestRegistrations (
    RegistrationID INT IDENTITY(1,1) PRIMARY KEY,
    GuestID INT NOT NULL,
    SharedExperienceID INT NULL,
    ExperienceID INT NULL,  
    FOREIGN KEY (GuestID) REFERENCES Guests(GuestID),
    FOREIGN KEY (SharedExperienceID) REFERENCES SharedExperiences(SharedExperienceID),
    FOREIGN KEY (ExperienceID) REFERENCES Experiences(ExperienceID)
);

CREATE TABLE ExperienceInSharedExperience (
    ID INT IDENTITY(1,1) PRIMARY KEY,  
    SharedExperienceID INT NOT NULL,  
    ExperienceID INT NOT NULL,
    FOREIGN KEY (SharedExperienceID) REFERENCES SharedExperiences(SharedExperienceID),
    FOREIGN KEY (ExperienceID) REFERENCES Experiences(ExperienceID)
);










-- ExperienceProviders
INSERT INTO ExperienceProviders (ProviderId, ProviderName, Address, PhoneNumber)
VALUES ('11111114', 'Noah’s Hotel', 'Finlandsgade 17, 8200 Aarhus N', '+45 71555080');

-- Experiences
INSERT INTO Experiences (ExperienceName, ExperienceProviderCVR, Price)
VALUES 
    ('Night at Noah''s Hotel Single room', '11111114', 730.50),
    ('Night at Noah''s Hotel Double room', '11111114', 910.99),
    ('Flight AAR - VIE', NULL, 1000.70),
    ('Vienna Historic Center Walking Tour', NULL, 100.00);

-- SharedExperiences
INSERT INTO SharedExperiences (ExperienceName, ExperienceDate)
VALUES 
    ('Trip to Austria', '2024-07-02'),
    ('Dinner Downtown', '2024-04-07'),
    ('Pottery Weekend', '2024-03-22');

-- Guests
INSERT INTO Guests (Name) 
VALUES 
    ('Joan'),
    ('Suzzane'),
    ('Patrick'),
    ('Anne');

-- GuestRegistrations
-- All are registerede for "Trip to Austria, but only Joan and Suzzane is registered for "Vienna Historic Center Walking Tour"
INSERT INTO GuestRegistrations (GuestID, SharedExperienceID, ExperienceID)
VALUES 
    (1, 1, 4),  
    (2, 1, 4),  
    (3, 1, NULL),  
    (4, 1, NULL);  

-- All guests are registered for "Flight AAR – VIE"
INSERT INTO GuestRegistrations (GuestID, SharedExperienceID, ExperienceID)
VALUES 
    (1, NULL, 3),  
    (2, NULL, 3), 
    (3, NULL, 3), 
    (4, NULL, 3); 

-- All guests are registerede for "Night at Noah’s Hotel Single room"
INSERT INTO GuestRegistrations (GuestID, SharedExperienceID, ExperienceID)
VALUES 
    (1, NULL, 1),
    (2, NULL, 1), 
    (3, NULL, 1), 
    (4, NULL, 1);  

-- ExperienceInSharedExperience
INSERT INTO ExperienceInSharedExperience (SharedExperienceID, ExperienceID)
VALUES 
    (1, 3),  -- "Flight AAR – VIE" is a part of "Trip to Austria"
    (1, 1),  -- "Night at Noah’s Hotel Single room" is a part of "Trip to Austria"
    (1, 4);  -- "Vienna Historic Center Walking Tour" is a part of "Trip to Austria"




-- 1. Get the data collected for each experience provider.
SELECT ExperienceProviders.Address, ExperienceProviders.PhoneNumber, ExperienceProviders.ProviderId
FROM ExperienceProviders

-- 2. List the experiences available in the system.
SELECT DISTINCT ExperienceName, 
       CONCAT(Price, ' DKK') AS PriceWithCurrency
FROM Experiences;

-- 3. Get the list of shared experiences and their date in the system in descending order
SELECT ExperienceName, ExperienceDate FROM SharedExperiences ORDER BY ExperienceDate DESC;

-- 4. Get the guests registered for a shared experience
SELECT DISTINCT g.Name 
FROM Guests g
JOIN GuestRegistrations gr ON g.GuestID = gr.GuestID
JOIN SharedExperiences se ON gr.SharedExperienceID = se.SharedExperienceID
WHERE se.ExperienceName = 'Trip to Austria';

-- 5. Get the experiences included in a shared experience:
SELECT DISTINCT e.ExperienceName
FROM ExperienceInSharedExperience eis
JOIN Experiences e ON eis.ExperienceID = e.ExperienceID
JOIN SharedExperiences se ON eis.SharedExperienceID = se.SharedExperienceID
WHERE se.ExperienceName = 'Trip to Austria';

-- 6. Get the guests registered for one of the experiences in a shared experience.
SELECT DISTINCT g.Name
FROM GuestRegistrations gr
JOIN Guests g ON gr.GuestID = g.GuestID
JOIN SharedExperiences se ON gr.SharedExperienceID = se.SharedExperienceID
JOIN Experiences e ON gr.ExperienceID = e.ExperienceID
WHERE se.ExperienceName = 'Trip to Austria' 
AND e.ExperienceName = 'Vienna Historic Center Walking Tour';

-- 7. Get the minimum, average, and maximum price for the whole experience in the system.
SELECT MIN(price) AS min_price, AVG(price) AS avg_price, MAX(price) AS max_price 
FROM Experiences;

-- 8. Get the number of guests and sum of sales for each experience available in the system.
SELECT 
    eo.ExperienceName, 
    COUNT(DISTINCT ger.GuestID) AS NumberOfGuests, 
    COALESCE(COUNT(DISTINCT ger.GuestID) * MAX(eo.Price), 0) AS TotalSales
FROM Experiences eo
LEFT JOIN GuestRegistrations ger ON eo.ExperienceID = ger.ExperienceID
GROUP BY eo.ExperienceName
ORDER BY 
    CASE 
        WHEN eo.ExperienceName = 'Night at Noah''s Hotel Single room' THEN 1
        WHEN eo.ExperienceName = 'Night at Noah''s Hotel Double room' THEN 2
        WHEN eo.ExperienceName = 'Flight AAR - VIE' THEN 3
        WHEN eo.ExperienceName = 'Vienna Historic Center Walking Tour' THEN 4
    END;

-- 9. One query of your choice to satisfy the informal requirements described above
-- Discount prices from Noah's Hotel
SELECT 
    e.ExperienceName AS IndividualExperience, 
    e.Price,
    COUNT(gr.GuestID) AS NumberOfGuests, 
        e.Price, 
        -- Price per person with 10% discount
    e.Price * 0.90 AS DiscountedPrice10,
    -- Price per person with 20% discount
    e.Price * 0.80 AS DiscountedPrice20
FROM Experiences e
LEFT JOIN GuestRegistrations gr ON e.ExperienceID = gr.ExperienceID
LEFT JOIN ExperienceProviders ep ON e.ExperienceProviderCVR = ep.ProviderId 
WHERE ep.ProviderId = '11111114' 
GROUP BY e.ExperienceName, e.Price
ORDER BY e.ExperienceName;


-- Experiences in SharedExperiences 
SELECT 
    se.SharedExperienceID, 
    se.ExperienceName AS SharedExperienceName, 
    e.ExperienceName AS IndividualExperience, 
    e.Price
FROM SharedExperiences se
LEFT JOIN ExperienceInSharedExperience esi ON se.SharedExperienceID = esi.SharedExperienceID
LEFT JOIN Experiences e ON esi.ExperienceID = e.ExperienceID
ORDER BY se.SharedExperienceID, e.ExperienceName;
