using Database;
using WebAPI.Models;

namespace WebAPI.Services;

public class ExperienceService
{
    private readonly DatabaseContext _context;

    public ExperienceService(DatabaseContext context)
    {
        _context = context;
    }

    public List<Experience> GetAll()
    {
        return _context.Experiences.ToList();
    }

    public Experience GetExperienceById(int id)
    {
        return _context.Experiences.Find(id)!;
    }

    // Get all experiences in a specific shared experience
    public List<Experience> GetExperiencesInSharedExperience(string sharedExperienceName)
    {
        return _context.Experiences
            .Where(g => g.SharedExperiences.Any(se => se.Name == sharedExperienceName))
            .ToList();
    }

    // Get the minimum, average, and maximum price for the whole experience in the system.
    public PriceStatistics GetPriceStatistics()
    {
        var minPrice = _context.Experiences.Min(e => e.Price);
        var avgPrice = _context.Experiences.Average(e => e.Price);
        var maxPrice = _context.Experiences.Max(e => e.Price);

        return new PriceStatistics
        {
            MinPrice = minPrice,
            AvgPrice = (decimal)avgPrice,
            MaxPrice = maxPrice
        };
    }

    // Get the number of guests and sum of sales for each experience available in the system.
    public List<ExperienceSales> GetExperienceSales()
    {
        var experienceNames = _context.Experiences.Select(e => e.Name).ToList();

        List<ExperienceSales> experienceSales = new List<ExperienceSales>();

        foreach (var name in experienceNames)
        {
            var numberOfGuests = _context.Experiences
                .Where(e => e.Name == name)
                .Select(e => e.Guests.Count)
                .FirstOrDefault();

            var totalSales = _context.Experiences
                .Where(e => e.Name == name)
                .Select(e => e.Guests.Count() * e.Price)
                .FirstOrDefault();

            experienceSales.Add(new ExperienceSales
            {
                ExperienceName = name,
                NumberOfGuests = numberOfGuests,
                TotalSales = totalSales
            });
        }
        return experienceSales;
    }


    //CRUD-operations from assignment
    //1. Add a new experience.
    public Provider GetProviderByName(string providerName)
    {
        return _context.Providers.FirstOrDefault(p => p.Name == providerName)!;
    }

    public Experience AddExperience(Experience experience)
    {
        _context.Experiences.Add(experience);
        _context.SaveChanges();
        return experience;
    }
    //2. Update the price of one of the experiences.
    public Experience UpdateExperiencePrice(int id, int newPrice)
    {
        var experience = _context.Experiences.Find(id);
        experience.Price = newPrice;
        _context.SaveChanges();
        return experience;
    }
    //3. Delete an experience
    public void DeleteExperience(int id)
    {
        var experience = _context.Experiences.Find(id);
        _context.Experiences.Remove(experience);
        _context.SaveChanges();
    }

}
public class PriceStatistics
{
    public decimal MinPrice { get; set; }
    public decimal AvgPrice { get; set; }
    public decimal MaxPrice { get; set; }
}

public class ExperienceSales
{
    public string ExperienceName { get; set; }
    public int NumberOfGuests { get; set; }
    public decimal TotalSales { get; set; }
}
