using System.Text;
using Assignment2_EFCore;
using Database;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.Identity;
using Microsoft.EntityFrameworkCore;
using Microsoft.IdentityModel.Tokens;
using Microsoft.OpenApi.Models;
using WebAPI.Models;
using WebAPI.Services;

public static class StartupManager
{
    /// <summary>
    /// Configures all services required by the application.
    /// </summary>
    public static void ConfigureServices(IServiceCollection services, IConfiguration configuration)
    {
        ConfigureDatabase(services, configuration);
        ConfigureIdentity(services);
        ConfigureAuthentication(services, configuration);
        ConfigureAuthorization(services);
        ConfigureSwagger(services);
        RegisterApplicationServices(services);

        // Add MVC controllers
        services.AddControllers();
    }

    /// <summary>
    /// Configures the database context using the connection string.
    /// </summary>
    private static void ConfigureDatabase(IServiceCollection services, IConfiguration configuration)
    {
        var conn = configuration["ConnectionStrings:DefaultConnection"];
        services.AddDbContext<DatabaseContext>(options =>
        {
            options.UseSqlServer(conn);
        });
    }

    /// <summary>
    /// Configures ASP.NET Identity for user authentication and management.
    /// </summary>
    private static void ConfigureIdentity(IServiceCollection services)
    {
        services.AddIdentity<ApplicationUser, IdentityRole>()
            .AddEntityFrameworkStores<DatabaseContext>()
            .AddDefaultTokenProviders();

        // Set password policy requirements
        services.Configure<IdentityOptions>(options =>
        {
            options.Password.RequireDigit = false;
            options.Password.RequiredLength = 4;
            options.Password.RequireLowercase = false;
            options.Password.RequireUppercase = false;
            options.Password.RequireNonAlphanumeric = false;
        });
    }

    /// <summary>
    /// Configures JWT authentication using secret key and issuer.
    /// </summary>
    private static void ConfigureAuthentication(IServiceCollection services, IConfiguration configuration)
    {
        var jwtKey = configuration["Jwt:Key"];
        var jwtIssuer = configuration["Jwt:Issuer"];

        services.AddAuthentication(options =>
        {
            options.DefaultAuthenticateScheme = JwtBearerDefaults.AuthenticationScheme;
            options.DefaultChallengeScheme = JwtBearerDefaults.AuthenticationScheme;
        })
        .AddJwtBearer(options =>
        {
            options.RequireHttpsMetadata = false;
            options.SaveToken = true;
            options.TokenValidationParameters = new TokenValidationParameters
            {
                ValidateIssuer = true,
                ValidateAudience = true,
                ValidateLifetime = true,
                ValidateIssuerSigningKey = true,
                ValidIssuer = jwtIssuer,
                ValidAudience = jwtIssuer,
                IssuerSigningKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(jwtKey)),
            };
        });
    }

    /// <summary>
    /// Configures authorization policies for different user roles.
    /// </summary>
    private static void ConfigureAuthorization(IServiceCollection services)
    {
        services.AddAuthorization(options =>
        {
            options.AddPolicy("AdminPolicy", policy => policy.RequireRole("Admin"));
            options.AddPolicy("ManagerPolicy", policy => policy.RequireRole("Manager"));
        });
    }

    /// <summary>
    /// Configures Swagger for API documentation.
    /// </summary>
    private static void ConfigureSwagger(IServiceCollection services)
    {
        services.AddEndpointsApiExplorer();
        services.AddSwaggerGen(options =>
        {
            options.AddSecurityDefinition("Bearer", new OpenApiSecurityScheme
            {
                Description = "Please enter token",
                Name = "Authorization",
                In = ParameterLocation.Header,
                Type = SecuritySchemeType.Http,
                Scheme = "Bearer",
                BearerFormat = "JWT"
            });

            options.AddSecurityRequirement(new OpenApiSecurityRequirement
            {
                {
                    new OpenApiSecurityScheme
                    {
                        Reference = new OpenApiReference
                        {
                            Type = ReferenceType.SecurityScheme,
                            Id = "Bearer"
                        }
                    },
                    Array.Empty<string>()
                }
            });
        });
    }

    /// <summary>
    /// Registers application-specific services for dependency injection.
    /// </summary>
    private static void RegisterApplicationServices(IServiceCollection services)
    {
        services.AddScoped<GuestService>();
        services.AddScoped<ProviderService>();
        services.AddScoped<SharedExperienceService>();
        services.AddScoped<ExperienceService>();
    }

    /// <summary>
    /// Seeds initial data, including roles and users, into the database.
    /// </summary>
    public static async Task SeedDataAsync(IServiceProvider services)
    {
        using var scope = services.CreateScope();
        var serviceProvider = scope.ServiceProvider;
        var context = serviceProvider.GetRequiredService<DatabaseContext>();
        var userManager = serviceProvider.GetRequiredService<UserManager<ApplicationUser>>();
        var roleManager = serviceProvider.GetRequiredService<RoleManager<IdentityRole>>();

        await SeedData.SeedRolesAndUsers(userManager, roleManager);

        var seedData = new SeedData();
        seedData.Seed(context);
    }

    /// <summary>
    /// Configures middleware for request processing and API documentation.
    /// </summary>
    public static void ConfigureMiddleware(WebApplication app)
    {
        if (app.Environment.IsDevelopment())
        {
            app.UseSwagger();
            app.UseSwaggerUI();
        }

        app.UseHttpsRedirection();
        app.UseAuthentication();
        app.UseAuthorization();
        app.MapControllers();
    }
}
